# copy me in elisa trunk before run please
from setuptools import setup, find_packages
import pkg_resources
import sys
import os
import shutil
import inspect
from distutils import dir_util

sys.path.insert(0, os.getcwd())

from elisa.core.utils.dist import scan_plugin_conf
from elisa.core import __version__


# copied from setup.py
ENTRY_POINTS="""\
      
      # Good Bundle
      services_good = elisa.plugins.good.services_good
      theme_switcher = elisa.plugins.good.theme_switcher
      aspect_ratio_switcher = elisa.plugins.good.aspect_ratio_switcher
      input_good = elisa.plugins.good.input_good
      media_good = elisa.plugins.good.media_good
      audiocd = elisa.plugins.good.audiocd
      fspot = elisa.plugins.good.fspot
      hal = elisa.plugins.good.hal
      pigment = elisa.plugins.good.pigment.pigment:PigmentPlugin
      testing = elisa.plugins.good.testing

      # Ugly Bundle
      flickr = elisa.plugins.ugly.flickr
      media_ugly = elisa.plugins.ugly.media_ugly
      coherence_plugin = elisa.plugins.ugly.coherence_plugin

      # Bad Bundle
      gtk = elisa.plugins.bad.gtk_frontend
      poblenou = elisa.plugins.bad.poblenou_frontend
      dvd = elisa.plugins.bad.dvd
      stage6 = elisa.plugins.bad.stage6
      weather = elisa.plugins.bad.weather
      input_bad = elisa.plugins.bad.input_bad
      media_bad = elisa.plugins.bad.media_bad
      rhythmbox = elisa.plugins.bad.rhythmbox
      
      service = elisa.plugins.service
"""

sys.argv.append('bdist_egg')

def build_egg_from_config(plugin_conf):

    # clear distutils' path cache
    dir_util._path_created = {}
    
    plugin_dir = os.path.dirname(plugin_conf)

    root_package = plugin_dir.replace(os.path.sep, '.')

    package_name = os.path.basename(plugin_dir)
    package_path = os.path.dirname(plugin_dir)
    
    namespace = package_path.replace(os.path.sep, '.')
    
    infos = scan_plugin_conf(plugin_conf)
    
    i = 0
    packages = []
    names = root_package.split('.')
    while i < len(names):
        if i == 0:
            s = names[0]
        else:
            s = '.'.join(names[:i+1])
        packages.append(s)
        i += 1

    infos['packages'] = packages
    infos['package_dir'] = {'': '.'}
    infos['package_data'] = {'': ['*.conf', '*.png', '*.mo', '*.po']}
    infos['namespace_packages'] = [namespace,]
    
    sub_packages = [ "%s.%s" % (root_package, pkg_name)
                     for pkg_name in find_packages(plugin_dir) ]
    infos['packages'].extend(sub_packages)
    
    #print infos['packages']
    setup(**infos)

def build_elisa_egg():

    packages = find_packages(exclude=('elisa.plugins.*',))

    base_path = os.path.join('elisa','plugins','base')
    base_packages = [ 'elisa.plugins.base.%s' % pkg
                      for pkg in find_packages(base_path)]
    base_packages.insert(0, 'elisa.plugins.base')

    packages.extend(base_packages)
    #print packages
    
    setup(name="elisa-core",
          version=__version__,
          description="""Fluendo Media Center""",
          author="The Elisa Team",
          author_email="",
          maintainer="Philippe Normand",
          maintainer_email="philippe@fluendo.com",
          long_description="""\
Elisa is an open media center system to watch, listen, manage and
share videos, files and music. It uses GStreamer, Python and OpenGL
technologies. """,
          url="http://elisa.fluendo.com",
          keywords=['multimedia', 'gstreamer', 'opengl'],
          classifiers = ['Development Status :: 3 - Alpha',
                         'Environment :: X11 Applications',
                         'Intended Audience :: End Users/Desktop',
                         'Natural Language :: English',
                         'Operating System :: OS Independent',
                         'Programming Language :: Python',
                         'Topic :: Multimedia :: Graphics :: 3D Rendering',
                         'Topic :: Multimedia :: Sound/Audio',
                         'Topic :: Multimedia :: Video',
                         ],
          packages=packages,
          package_data = {'elisa.plugins.base': ['*.conf',],
                          'elisa.plugins.base.themes': ['*.conf',],
                          },
          entry_points="""\
          [elisa.plugins]
          base = elisa.plugins.base

          [gui_scripts]
          elisa = elisa.core.application:main

          [distutils.commands]
          build_po = elisa.core.utils.dist:build_po
      
          [distutils.setup_keywords]
          elisa_infos = elisa.core.utils.dist:assert_dict_or_none

          [egg_info.writers]
          elisa_infos.txt = elisa.core.utils.dist:write_dict
          """
          )
    
def main(args):
    shutil.rmtree('build', ignore_errors=True)
    build_elisa_egg()

    plugins = pkg_resources.EntryPoint.parse_group('elisa.plugins',
                                                   ENTRY_POINTS)

    for plugin_name, entry_point in plugins.iteritems():
        shutil.rmtree('build', ignore_errors=True)
        
        plugin_path = entry_point.module_name.replace('.', os.path.sep)
        if not os.path.exists(plugin_path):
            cls = entry_point.load(require=False)
            plugin_path = os.path.dirname(inspect.getsourcefile(cls))
            cur_dir = os.path.abspath('.')
            plugin_path = plugin_path[len(cur_dir)+1:]
            
        plugin_conf = os.path.join(plugin_path, 'plugin.conf')
        if os.path.exists(plugin_conf):
            build_egg_from_config(plugin_conf)
            shutil.rmtree('elisa_plugin_%s.egg-info' % plugin_name,
                          ignore_errors=True)
        else:
            print '%s not found. Skipping Egg build of this plugin' % plugin_conf

if __name__ == '__main__':
    main(sys.argv[1:])
