# Elisa - Home multimedia server
# Copyright (C) 2006 Fluendo, S.A. (www.fluendo.com).
# All rights reserved.
# 
# This software is available under three license agreements.
# 
# There are various plugins and extra modules for Elisa licensed
# under the MIT license. For instance our upnp module uses this license.
# 
# The core of Elisa is licensed under GPL version 3.
# See "LICENSE.GPL" in the root of this distribution including a special 
# exception to use Elisa with Fluendo's plugins.
# 
# The GPL part is also available under a commerical licensing
# agreement.
# 
# The second license is the Elisa Commercial License Agreement.
# This license agreement is available to licensees holding valid
# Elisa Commercial Agreement licenses.
# See "LICENSE.Elisa" in the root of this distribution.

import sys
import os, platform

if platform.system() == 'Windows' and os.environ.has_key('ELISA_DEPS'):
    #Load the developement environement for windows
    from win32 import win_set_env

from elisa.core import __version__, log

from elisa.core.utils.dist import build_po
from elisa.core.utils.i18n import compile_translations_from_file

import setuptools
from pkg_resources import *
from setuptools import setup, find_packages, Extension

from setuptools.command import test
from setuptools.command import install
from setuptools.command import sdist
from setuptools.command import build_py
from setuptools.command import build_ext
from distutils.command import clean

from twisted.scripts import trial

try:
    import py2exe
except:
    pass

log.init()

def override_check_test_suite(dist, attr, value):
    if not isinstance(value, list):
        raise DistutilsSetupError("test_suite must be a list")
    
setuptools.dist.check_test_suite = override_check_test_suite

class SDist(sdist.sdist):

    def make_distribution(self):
        files = self.filelist.files[:]
        for filename in files:
            dummy, ext = os.path.splitext(filename)
            ext = ext[1:].lower()
            if ext == 'mo':
                self.filelist.files.remove(filename)
                self.announce("Removed mo file: %r" % filename)
        sdist.sdist.make_distribution(self)

class LocaleBuildPy(build_py.build_py):
    """
    create .mo files from .po locales if needed
    """
 
    def run(self):
        pobuild = build_po(self.distribution)
        pobuild.finalize_options()
        pobuild.run()
        build_py.build_py.run(self)
        
class TrialTest(test.test):
    """
    Twisted Trial setuptools command
    """

    user_options = test.test.user_options + [
        ('coverage','c', "Report coverage data"),
    ]

    def initialize_options(self):
        test.test.initialize_options(self)
        self.coverage = None

    def finalize_options(self):
        if self.test_suite is None:
            if self.test_module is None:
                self.test_suite = self.distribution.test_suite
            else:
                self.test_suite = self.test_module
        elif self.test_module:
            raise DistutilsOptionError(
                "You may specify a module or a suite, but not both"
            )

        self.test_args = self.test_suite

    def run_tests(self):
        config = trial.Options()
        config.parseOptions()

        args = self.test_args
        if type(args) == str:
            args = [args,]

        config['tests'] = args

        if self.coverage:
            config.opt_coverage()

        trial._initialDebugSetup(config)
        trialRunner = trial._makeRunner(config)
        suite = trial._getSuite(config)

        # run the tests
        if config['until-failure']:
            test_result = trialRunner.runUntilFailure(suite)
        else:
            test_result = trialRunner.run(suite)

        # write coverage data
        if config.tracer:
            sys.settrace(None)
            results = config.tracer.results()
            results.write_results(show_missing=1, summary=False,
                                  coverdir=config.coverdir)

class UriInterfaceExtension(Extension):
    def __init__(self, module_name, module_file, *args, **kw):
        Extension.__init__(self, module_name, [], *args, **kw)
        self.module_file = module_file

class BuildExt(build_ext.build_ext):
    def swig_sources(self, sources, ext):
        new_sources = build_ext.build_ext.swig_sources(self, sources, ext)

        if isinstance(ext, UriInterfaceExtension):
            # generate the source from the uri interface template
            template = file('data/gst_uri_interface_wrapper.c')
            fo = file(ext.module_file, "w")
            module_name = '_' + os.path.basename(ext.module_file)[:-2]
            fo.write(template.read() % {'module_name': module_name})
            fo.close()
            template.close()
            new_sources.append(ext.module_file)

        return new_sources

class Clean(clean.clean):
    def run(self):
        clean.clean.run(self)

        global c_extensions
        for ext in c_extensions:
            if not isinstance(ext, UriInterfaceExtension):
                continue

            filename = ext.module_file
            soname = os.path.splitext(filename)[0]
            dirname, basename = os.path.split(soname)
            try:
                os.unlink(filename)
                # we build in place so we need to remove the .so file
                os.unlink(os.path.join(dirname, '_' + basename + '.so'))
            except OSError:
                pass

data_files = []
c_extensions = []
options = {}

if platform.system() == 'Linux':
    import gst
    import subprocess

    data_files.extend([('share/pixmaps', ['data/elisa.png']),
                       ('share/applications', ['data/elisa.desktop'])])

    if gst.pygst_version <= (0, 10, 8):
        # GstURIHandler is broken, use our wrapper
        class PkgConfigError(Exception):
            pass

        try:
            pkgconfig = subprocess.Popen("pkg-config --libs pygobject-2.0 "
                                         "gstreamer-0.10",
                                         stdout=subprocess.PIPE, shell=True)
            if pkgconfig.wait() != 0:
                raise PkgConfigError()
            
            extra_link_args = pkgconfig.stdout.read().split()
            
            pkgconfig = subprocess.Popen("pkg-config --cflags pygobject-2.0 "
                                         "gstreamer-0.10",
                                         stdout=subprocess.PIPE, shell=True)
            if pkgconfig.wait() != 0:
                raise PkgConfigError()
            
            extra_compile_args = pkgconfig.stdout.read().split()

            module_name = 'elisa.plugins.bad.daap_plugin._daap_uri_interface'
            module_file = 'elisa/plugins/bad/daap_plugin/daap_uri_interface.c'
            ext = UriInterfaceExtension(module_name, module_file,
                                        extra_compile_args=extra_compile_args,
                                        extra_link_args=extra_link_args)
            c_extensions.append(ext)
        except PkgConfigError:
            # pygobject not installed, we can't compile our wrapper
            pass
        
elif platform.system() == 'Windows':
    module_include = ['tarfile', 'md5']
    package_include = ['distutils', 'win32']
    module_exclude = ['gst', 'pygst', 'elisa', 'twisted', 'pgm', 'gtk',
                      'gobject']
    dll_exclude = ['libglade-2.0-0.dll']

    options = {"custom_boot_script" : 'win32/win_set_env.py', 
               "includes"           : module_include, 
               "packages"           : package_include, 
               "excludes"           : module_exclude,
               "dll_excludes"       : dll_exclude
           } 

setup(cmdclass={'test': TrialTest, 'build_py': LocaleBuildPy,
                'build_ext': BuildExt, 'clean': Clean, 'sdist': SDist},
      data_files=data_files,
      name="elisa",
      console=['elisa.py'],
      options = {"py2exe": options},
      version=__version__,
      description="""Fluendo Media Center""",
      author="The Elisa Team",
      author_email="",
      maintainer="Philippe Normand",
      maintainer_email="philippe@fluendo.com",
      long_description="""\
Elisa is an open media center system to watch, listen, manage and
share videos, files and music. It uses GStreamer, Python and OpenGL
technologies. """,
      url="http://elisa.fluendo.com",
      keywords=['multimedia', 'gstreamer', 'opengl'],
      classifiers = ['Development Status :: 3 - Alpha',
                     'Environment :: X11 Applications',
                     'Intended Audience :: End Users/Desktop',
                     'Natural Language :: English',
                     'Operating System :: OS Independent',
                     'Programming Language :: Python',
                     'Topic :: Multimedia :: Graphics :: 3D Rendering',
                     'Topic :: Multimedia :: Sound/Audio',
                     'Topic :: Multimedia :: Video',
                     ],
      test_suite = ['elisa',],
      zip_safe = False,
      packages=find_packages(exclude=('win32*','external_plugins*')),
      include_package_data = True,
      ext_modules=c_extensions,
      entry_points="""
      [gui_scripts]
      elisa = elisa.core.application:main

      [elisa.plugins]
      base = elisa.plugins.base
      
      # Good Bundle
      lirc = elisa.plugins.good.lirc_plugin
      theme_switcher = elisa.plugins.good.theme_switcher
      aspect_ratio_switcher = elisa.plugins.good.aspect_ratio_switcher
      gvfs = elisa.plugins.good.gvfs_plugin
      audiocd = elisa.plugins.good.audiocd
      fspot = elisa.plugins.good.fspot
      hal = elisa.plugins.good.hal
      media_db = elisa.plugins.good.media_db
      pigment = elisa.plugins.good.pigment.pigment:PigmentPlugin
      testing = elisa.plugins.good.testing
      httpd = elisa.plugins.good.httpd_plugin
      twisted = elisa.plugins.good.twisted_plugin
      amazon = elisa.plugins.good.amazon_plugin
      gstreamer = elisa.plugins.good.gstreamer_plugin
      xmlmenu = elisa.plugins.good.xmlmenu
      
      # Ugly Bundle
      aggregator = elisa.plugins.ugly.aggregator_plugin
      flickr = elisa.plugins.ugly.flickr
      youtube = elisa.plugins.ugly.youtube
      stage6 = elisa.plugins.ugly.stage6
      shoutcast = elisa.plugins.ugly.shoutcast_plugin
      lastfm = elisa.plugins.ugly.lastfm_plugin
      
      # Bad Bundle
      gtk = elisa.plugins.bad.gtk_frontend
      helper = elisa.plugins.bad.helper_frontend
      poblenou = elisa.plugins.bad.poblenou_frontend
      raval = elisa.plugins.bad.raval_frontend
      dvd = elisa.plugins.bad.dvd
      weather = elisa.plugins.bad.weather
      webtv = elisa.plugins.bad.webtv_plugin
      daap = elisa.plugins.bad.daap_plugin
      ipod = elisa.plugins.bad.ipod_plugin
      rhythmbox = elisa.plugins.bad.rhythmbox
      osso = elisa.plugins.bad.osso 
      webcam = elisa.plugins.bad.webcam_plugin
      bluetooth = elisa.plugins.bad.bluetooth_plugin
      album_art = elisa.plugins.bad.album_art_plugin
      gnome = elisa.plugins.bad.gnome_plugin
      coherence = elisa.plugins.bad.coherence_plugin
      
      service = elisa.plugins.service
      
      [distutils.commands]
      build_po = elisa.core.utils.dist:build_po
      
      [distutils.setup_keywords]
      elisa_infos = elisa.core.utils.dist:assert_dict_or_none

      [egg_info.writers]
      elisa_infos.txt = elisa.core.utils.dist:write_dict
      """,
    )
